/***************************************************************************
 *
 * MODULE         : AM128SIO.C
 * Author/s       : RAV
 * Tested by      : RAV
 * Creation Date  : 20.11.2006
 *
 * Copyright      : (c) SPECTRO A. I. GmbH 2006
 *
 * ----------------------------------------------------------------------  *
 * Project        : 
 *
 * Version:       : 1.00
 *               
 * Description    : This module initializes
 *                  and makes SIO transmission and reception possible
 *                  for both serial ports
 *
 * Specials       : ISO Reference Model: Layer 1
 * System         : Atmel ATmega128
 *
 * Open Points    : 
 *
 *-------------------------------------------------------------------------*
 *
 * Name:      Vers.  Date:       History of Changes :
 *
 * RAV        1.00   20.11.2006  First Implementation
 *
 ***************************************************************************/

#include <AM128CTL.H>  
#include <AM128SIO.H>
//#include <DSP_DEF.H>    /* Global structures and defines................... */

/***************************************************************************
 * Project specific defines
 ***************************************************************************/
#ifdef GC_FIRST
  #include <FIRST.H>  
#endif

//#ifdef SINGLE_SIO
//  #define MAX_SIOS  1
//#else
  #define MAX_SIOS  2
//#endif

/***************************************************************************
 * Global Variables
 ***************************************************************************/
   
volatile  unsigned char acWritePointer[MAX_SIOS];
volatile  unsigned char acReadPointer[MAX_SIOS];  
volatile  unsigned char acBufferState[MAX_SIOS];

volatile  unsigned char acRecDataCtrl_0[REC_BUFFER_LGTH];
volatile  unsigned char acSIO_0_Buffer[REC_BUFFER_LGTH];

//#ifndef SINGLE_SIO
//volatile  unsigned char acRecDataCtrl_1[REC_BUFFER_LGTH];
//volatile  unsigned char acSIO_1_Buffer[REC_BUFFER_LGTH];
//#endif


/***************************************************************************
 *
 * FUNCTION      : SYS_InitCom
 * Author/s      : RAV
 * Tested by     : RAV
 * Creation Date : 20.11.2006
 *
 * Copyright     : (c) SPECTRO A. I. GmbH 2006
 *
 * ----------------------------------------------------------------------  *
 * Project       : 
 *
 * Version:      : 1.00
 *
 * Description   : Initialisizes the Serial Ports 0 and 1 (UART)
 *
 * System        : Atmel ATmega128
 *
 *-------------------------------------------------------------------------*
 *
 * Name:      Vers.  Date:       History of Changes :
 *
 * RAV        1.00   20.11.2006  First Implementation
 ***************************************************************************/
unsigned char SYS_InitCom(unsigned char ucCom, unsigned char ucParity, unsigned char ucDatabits, long lBaudRate)
{ 
  unsigned char i;

  switch (ucCom)                                          /* SIO to be initialized */
  {
    case IM_MB_SIO_0:                                     /* SIO 0 */
      acWritePointer[ucCom]  = 0;
      acReadPointer[ucCom]   = 0;
      acBufferState[ucCom]   = REC_BUFFER_EMPTY;
    
      for (i = 0; i < REC_BUFFER_LGTH; i++)               /* Initialize Receive-Buffer  */
      {
        acSIO_0_Buffer[i] = 0x00;
        acRecDataCtrl_0[i] = SIO_NO_ERROR;
      }

      switch(ucParity)                                    /* Set parity                 */
      {
	    case IM_MB_NO_PARITY:
		  SEL_SIO_0_NO_PARITY;
		  break;
        case IM_MB_EVEN_PARITY:
		  SEL_SIO_0_EVEN_PARITY;
		  break;
        case IM_MB_ODD_PARITY:
		  SEL_SIO_0_ODD_PARITY;
          break;
        default:
          return (IDE_INITFAILURE);
      }

      switch(ucDatabits)                                  /* Set data length            */
      {
        case IM_MB_7_DATABITS:
		  SEL_SIO_0_7_BIT_DATA;
          break;
        case IM_MB_8_DATABITS:
		  SEL_SIO_0_8_BIT_DATA;
          break;
        default:
          return (IDE_INITFAILURE);
      }

      SEL_SIO_0_1_STOPBIT;
      SEL_SIO_0_ASYNC_MODE;
      SEL_SIO_0_DEF_SPEED;
      SET_SIO_0_BR_REG((unsigned int)(SYS_CLOCK / (16L * lBaudRate) - 1));
	  ENABLE_SIO_0_TRM_INT;
      ENABLE_SIO_0_REC_INT;
	  RESET_DE_0;
      break; /* case IM_MB_SIO_0 */

#ifndef SINGLE_SIO
    case IM_MB_SIO_1:
      acWritePointer[ucCom]  = 0;
      acReadPointer[ucCom]   = 0;
      acBufferState[ucCom]   = REC_BUFFER_EMPTY;
    
      for (i = 0; i < REC_BUFFER_LGTH; i++)               /* Initialize Receive-Buffer  */
      {
        acSIO_1_Buffer[i] = 0x00;
        acRecDataCtrl_1[i] = SIO_NO_ERROR;
      }
      
      switch(ucParity)                                    /* Set parity                 */
      {
	    case IM_MB_NO_PARITY:
		  SEL_SIO_1_NO_PARITY;
		  break;
        case IM_MB_EVEN_PARITY:
		  SEL_SIO_1_EVEN_PARITY;
		  break;
        case IM_MB_ODD_PARITY:
		  SEL_SIO_1_ODD_PARITY;
          break;
        default:
          return (IDE_INITFAILURE);
      }

      switch(ucDatabits)                                  /* Set data length            */
      {
        case IM_MB_7_DATABITS:
		  SEL_SIO_1_7_BIT_DATA;
          break;
        case IM_MB_8_DATABITS:
		  SEL_SIO_1_8_BIT_DATA;
          break;
        default:
          return (IDE_INITFAILURE);
      }

      SEL_SIO_1_1_STOPBIT;
      SEL_SIO_1_ASYNC_MODE;
      SEL_SIO_1_DEF_SPEED;
      SET_SIO_1_BR_REG((unsigned int)(SYS_CLOCK / (16L * lBaudRate) - 1));
	  ENABLE_SIO_1_TRM_INT;
      ENABLE_SIO_1_REC_INT;
	  RESET_DE_1;
      break; /* case (IM_MB_SIO_1) */
#endif

    default:
      return (IDE_INITFAILURE);
  }


  return (IDE_NO_ERROR);
}






/***************************************************************************
 *
 * FUNCTION      : SYS_Transmit
 * Author/s      : RAV
 * Tested by     : RAV
 * Creation Date : 20.11.2006
 *
 * Copyright     : (c) SPECTRO A. I. GmbH 2006
 *
 * ----------------------------------------------------------------------  *
 * Project       : 
 *
 * Version:      : 1.00
 *
 * Description   : Transmits one data byte via the specified port
 *
 * System        : Atmel ATmega128
 *
 *-------------------------------------------------------------------------*
 *
 * Name:      Vers.  Date:       History of Changes :
 *
 * RAV        1.00   20.11.2006  First Implementation
 ***************************************************************************/
unsigned char SYS_Transmit(unsigned char ucCom, unsigned char ucData)
{ 
  unsigned int wTimeoutcounter = 0;                                      

  switch (ucCom)
  {
    case IM_MB_SIO_0:
      while (DE_0_ACTIVE)                            /* Wait for Transmitter to be ready  */
      {
        wTimeoutcounter++;                           /* ................................. */
        if (wTimeoutcounter >= TRM_SYS_COUNTER)      /* Timeoutcounter max? ............. */
          return (IDE_TRANSMIT_TIMEOUT);
      }
      SET_DE_0; 
      TRANSMIT_SIO_0_DATA(ucData);                   /* Write ucData -> Serial Databuffer */
      break;

#ifndef SINGLE_SIO
    case IM_MB_SIO_1:  
      while (DE_1_ACTIVE)                            /* Wait for Transmitter to be ready  */
	  {
        wTimeoutcounter++;                           /* ................................. */
        if (wTimeoutcounter >= TRM_SYS_COUNTER)      /* Timeoutcounter max? ............. */
          return (IDE_TRANSMIT_TIMEOUT);
      }
      SET_DE_1; 
      TRANSMIT_SIO_1_DATA(ucData);                   /* Write ucData -> Serial Databuffer */
      break;
#endif
 
    default:;
  }

  return (IDE_NO_ERROR);
} /* SYS_Transmit */







/***************************************************************************
 *
 * FUNCTION      : SYS_Receive
 * Author/s      : RAV
 * Tested by     : RAV
 * Creation Date : 20.11.2006
 *
 * Copyright     : (c) SPECTRO A. I. GmbH 2006
 *
 * ----------------------------------------------------------------------  *
 * Project       : 
 *
 * Version:      : 1.00
 *
 * Description   : Reads one data byte of the specified port from the 
 *                 receive buffer
 *
 * System        : Atmel ATmega128
 *
 *-------------------------------------------------------------------------*
 *
 * Name:      Vers.  Date:       History of Changes :
 *
 * RAV        1.00   20.11.2006  First Implementation
 ***************************************************************************/
unsigned char SYS_Receive(unsigned char ucCom, unsigned char *pcData)
{
  unsigned char ucSIOError = 0;

  if (acBufferState[ucCom] != REC_BUFFER_EMPTY)               /* Data available?           */
  {
    switch(ucCom)                                             /* Read data from buffer     */
    {
      case IM_MB_SIO_0:
        *pcData    = acSIO_0_Buffer[acReadPointer[ucCom]];
        ucSIOError = acRecDataCtrl_0[acReadPointer[ucCom]];
        break;
#ifndef SINGLE_SIO
      case IM_MB_SIO_1:
        *pcData    = acSIO_1_Buffer[acReadPointer[ucCom]];
        ucSIOError = acRecDataCtrl_1[acReadPointer[ucCom]];
        break;
#endif
      default:;
    }

    if (acReadPointer[ucCom] < (REC_BUFFER_LGTH - 1))         /* Shift ReadPointer         */
      acReadPointer[ucCom]++;
    else
      acReadPointer[ucCom] = 0;

    
    DISABLE_INTERRUPTS;                            /* Disable Interrupts to prevent data loss */

    if (acReadPointer[ucCom] == acWritePointer[ucCom])        /* Still data available?     */
      acBufferState[ucCom] = REC_BUFFER_EMPTY;
    else
      acBufferState[ucCom] = REC_BUFFER_READY;

    ENABLE_INTERRUPTS;                                             /* Re-enable Interrupts   */

    if ((ucSIOError & SIO_PARITY_ERROR) == SIO_PARITY_ERROR)       /* Parity-error occured?  */ 
      return (IDE_PARITY_ERROR);
    else
    if ((ucSIOError & SIO_FRAME_ERROR) == SIO_FRAME_ERROR)         /* Frame-error occured?   */ 
      return (IDE_FRAME_ERROR);
    else
    if ((ucSIOError & SIO_OVERRUN_ERROR) == SIO_OVERRUN_ERROR)     /* Overrun-error occured? */ 
      return (IDE_RECEIVE_FAIL);
    else
      return (IDE_NO_ERROR);                                  /* Data is valid             */

  }
  else
    return (IDE_RECEIVE_TIMEOUT);                             /* No data available                 */

} /* SYS_Receive */





/***************************************************************************
 *
 * FUNCTION      : SYS_ClearRecBuffer
 * Author/s      : RAV
 * Tested by     : RAV
 * Creation Date : 20.11.2006
 *
 * Copyright     : (c) SPECTRO A. I. GmbH 2006
 *
 * ----------------------------------------------------------------------  *
 * Project       : 
 *
 * Version:      : 1.00
 *
 * Description   : clears the specified receive buffer
 *
 * System        : Atmel ATmega128
 *
 *-------------------------------------------------------------------------*
 *
 * Name:      Vers.  Date:       History of Changes :
 *
 * RAV        1.00   20.11.2006  First Implementation
 ***************************************************************************/
void SYSClearRecBuffer(unsigned char ucCom)
{
  DISABLE_INTERRUPTS;                            /* Disable Interrupts to prevent data loss */

  acWritePointer[ucCom]  = 0;
  acReadPointer[ucCom]   = 0;
  acBufferState[ucCom]   = REC_BUFFER_EMPTY;

  ENABLE_INTERRUPTS;                                      /* Re-enable Interrupts   */
}









#ifdef pilze
/**************************************************************************
 *
 * FUNCTION      : ISR(USART0_T_vect)
 * Author/s      : RAV
 * Tested by     : RAV
 * Creation Date : 20.11.2006
 *
 * Copyright     : (c) SPECTRO A. I. GmbH 2006
 *
 * ----------------------------------------------------------------------  *
 * Project       : 
 *
 * Version:      : 1.00
 *
 * Description   : ISR for SIO 0 transmit ready
 *
 * System        : Atmel ATmega128
 *
 *-------------------------------------------------------------------------*
 *
 * Name:      Vers.  Date:       History of Changes :
 *
 * RAV        1.00   20.11.2006  First Implementation
 ***************************************************************************/
ISR(USART0_TX_vect)
{
  RESET_DE_0;
}




/***************************************************************************
 *
 * FUNCTION      : ISR(USART0_RX_vect)
 * Author/s      : RAV
 * Tested by     : RAV
 * Creation Date : 20.11.2006
 *
 * Copyright     : (c) SPECTRO A. I. GmbH 2006
 *
 * ----------------------------------------------------------------------  *
 * Project       : 
 *
 * Version:      : 1.00
 *
 * Description   : ISR for SIO 0 receive buffer
 *
 * System        : Atmel ATmega128
 *
 *-------------------------------------------------------------------------*
 *
 * Name:      Vers.  Date:       History of Changes :
 *
 * RAV        1.00   20.11.2006  First Implementation
 ***************************************************************************/
ISR(USART0_RX_vect)
{
  if (acBufferState[IM_MB_SIO_0] != REC_BUFFER_FULL)         /* Free space available?             */
  {
    READ_SIO_0_ERRORS(acRecDataCtrl_0[acWritePointer[IM_MB_SIO_0]]);
    READ_SIO_0_DATA(acSIO_0_Buffer[acWritePointer[IM_MB_SIO_0]]);     

    if (acWritePointer[IM_MB_SIO_0] < (REC_BUFFER_LGTH - 1)) /* Shift WritePointer         */
      acWritePointer[IM_MB_SIO_0]++;
    else
      acWritePointer[IM_MB_SIO_0] = 0;

    if (acReadPointer[IM_MB_SIO_0] == acWritePointer[IM_MB_SIO_0]) /* Still space available?     */
      acBufferState[IM_MB_SIO_0] = REC_BUFFER_FULL;
    else
      acBufferState[IM_MB_SIO_0] = REC_BUFFER_READY;
  }
}
#endif               



#ifndef SINGLE_SIO
/**************************************************************************
 *
 * FUNCTION      : ISR(USART1_T_vect)
 * Author/s      : RAV
 * Tested by     : RAV
 * Creation Date : 20.11.2006
 *
 * Copyright     : (c) SPECTRO A. I. GmbH 2006
 *
 * ----------------------------------------------------------------------  *
 * Project       : 
 *
 * Version:      : 1.00
 *
 * Description   : ISR for SIO 1 transmit ready
 *
 * System        : Atmel ATmega128
 *
 *-------------------------------------------------------------------------*
 *
 * Name:      Vers.  Date:       History of Changes :
 *
 * RAV        1.00   20.11.2006  First Implementation
 ***************************************************************************/
ISR(USART1_TX_vect)
{
  RESET_DE_1;
}




/***************************************************************************
 *
 * FUNCTION      : ISR(USART1_RX_vect)
 * Author/s      : RAV
 * Tested by     : RAV
 * Creation Date : 20.11.2006
 *
 * Copyright     : (c) SPECTRO A. I. GmbH 2006
 *
 * ----------------------------------------------------------------------  *
 * Project       : 
 *
 * Version:      : 1.00
 *
 * Description   : ISR for SIO 1 receive buffer
 *
 * System        : Atmel ATmega128
 *
 *-------------------------------------------------------------------------*
 *
 * Name:      Vers.  Date:       History of Changes :
 *
 * RAV        1.00   20.11.2006  First Implementation
 ***************************************************************************/
ISR(USART1_RX_vect)
{
  if (acBufferState[IM_MB_SIO_1] != REC_BUFFER_FULL)         /* Free space available?             */
  {
    READ_SIO_1_ERRORS(acRecDataCtrl_1[acWritePointer[IM_MB_SIO_1]]);
    READ_SIO_1_DATA(acSIO_1_Buffer[acWritePointer[IM_MB_SIO_1]]);     

    if (acWritePointer[IM_MB_SIO_1] < (REC_BUFFER_LGTH - 1)) /* Shift WritePointer         */
      acWritePointer[IM_MB_SIO_1]++;
    else
      acWritePointer[IM_MB_SIO_1] = 0;

    if (acReadPointer[IM_MB_SIO_1] == acWritePointer[IM_MB_SIO_1]) /* Still space available?     */
      acBufferState[IM_MB_SIO_1] = REC_BUFFER_FULL;
    else
      acBufferState[IM_MB_SIO_1] = REC_BUFFER_READY;
  }
}
               



#endif
               



