#ifndef AM128SIO_H
#define AM128SIO_H
/***************************************************************************
 * INCLUDEFILE   : AM128SIO.H
 * to MODULE     : AM128SIO.C
 * Author        : RAV
 * Tested by     : RAV
 * Creation Date : 23.11-2006
 *
 * Copyright     : (c) SPECTRO A. I. GmbH 2006
 *
 *-------------------------------------------------------------------------*
 * Project       : 
 *
 * Version       : 1.00
 *
 * Description   : Headerfile to C550_SIO.C
 *
 * Specials      : ISO Reference Model: Layer 1
 * System        : Dallas 87C550
 *
 *-------------------------------------------------------------------------*
 *
 * Name:      Vers.  Date:       History of Changes :
 *
 * RAV        1.00   23.11.2006  First Implementation
 *
 ***************************************************************************/


#define IDE_NO_ERROR               0x00
#define IDE_INITFAILURE            0x09   /* Initialization of SIO failed ....... */
#define IDE_PARITY_ERROR           0x10   /* Receive error (Parity) ............. */
#define IDE_FRAME_ERROR            0x11   /* Receive error (Frame) .............. */
#define IDE_RECEIVE_TIMEOUT        0x12   /* Receive error (Timeout) ............ */
#define IDE_TRANSMIT_TIMEOUT       0x13   /* Transmit error (Timeout) ........... */
#define IDE_RECEIVE_FAIL           0x62   /* Failure during receipt ...........*/



#define  TRM_SYS_COUNTER          2000
#define  REC_BUFFER_READY         0
#define  REC_BUFFER_EMPTY         1
#define  REC_BUFFER_FULL          2
#define  SIO_NO_ERROR             0x00
#define  SIO_PARITY_ERROR         0x04
#define  SIO_OVERRUN_ERROR        0x08
#define  SIO_FRAME_ERROR          0x10


/*********************************************************************************
 * Modes for serial transmission
 *********************************************************************************/

#define IM_MB_NO_PARITY             2
#define IM_MB_ODD_PARITY            1
#define IM_MB_EVEN_PARITY           0
#define IM_MB_1_STOPBIT             1
#define IM_MB_2_STOPBITS            2
#define IM_MB_7_DATABITS            7
#define IM_MB_8_DATABITS            8
#define IM_MB_SIO_0                 0
#define IM_MB_SIO_1                 1
#define IM_MB_BAUDRATE_7200      7200
#define IM_MB_BAUDRATE_9600      9600
#define IM_MB_BAUDRATE_19200    19200
#define IM_MB_BAUDRATE_57600    57600
#define IM_MB_BAUDRATE_28800    28800
#define IM_MB_BAUDRATE_38400    38400
#define IM_MB_BAUDRATE_115200  115200


/******************************************************************************
 * Project specific defines
 ******************************************************************************/

#ifdef GC_FIRST
  #define SEL_DE_0                  SEL_PE2_OUTPUT
  #define SET_DE_0                  PORTE &= ~0x04             
  #define RESET_DE_0                PORTE |= 0x04
  #define DE_0_ACTIVE               ((PINE & 0x04) == 0x00)
  #define REC_BUFFER_LGTH           32
  #define SINGLE_SIO
#endif


/******************************************************
 * PROTOTYPES.........................................*
 ******************************************************/

//unsigned char  _Z11SYS_InitComhhhl(unsigned char,unsigned char,unsigned char,long);
//unsigned char  _Z12SYS_Transmithh(unsigned char,unsigned char);
unsigned char  SYS_InitCom(unsigned char,unsigned char,unsigned char,long);
unsigned char  SYS_Transmit(unsigned char,unsigned char);
unsigned char  SYS_Receive(unsigned char,unsigned char*);
void           SYS_ClearRecBuffer(unsigned char);

#endif












